(* *********************************************************************)
(*                                                                     *)
(*              The Verasco static analyzer                            *)
(*                                                                     *)
(*          Vincent Laporte, Université de Rennes 1                    *)
(*          Jacques-Henri Jourdan, INRIA Paris-Rocquencourt            *)
(*                                                                     *)
(*  Copyright Institut National de Recherche en Informatique et en     *)
(*  Automatique and Université de Rennes 1.  All rights reserved.      *)
(*  This file is distributed under the terms of the GNU General        *)
(*  Public License as published by the Free Software Foundatio,        *)
(*  either version 2 of the License, or (at your option) any later     *)
(*  version.  This file is also distributed under the terms of the     *)
(*  INRIA Non-Commercial License Agreement.                            *)
(*                                                                     *)
(* *********************************************************************)

Require Import
  Utf8
  Coqlib
  Globalenvs
  Csharpminor
  Behaviors
  String.

Require Import
  Utils
  ShareTree
  ToString
  AbMachineEnv
  IdealEnvToMachineEnv
  IdealIntervalsNonrel
  PolAdomZ
  AbLinearize
  Octagons
  AbTyps
  AlarmMon
  AbMemSignatureCsharpminor
  DebugAbMemCsharpminor
  CsharpminorIter
  CsharpminorIterproof
  Cells MemCsharpminor
  AbIdealEnvProduct
  AbVarExpEq
  Hash.

Require Import
  DebugAbMachineEnv
  ToString
  AdomLib
  ZCongruences
  CellEncoding.

Section S.

  Context (max_concretize : N)
          (trace verbose: bool)
          (unroll: nat).

  Context {tnum tnum_iter: Type}
          `{ToString tnum} `{ToString tnum_iter}
          (M: @ab_machine_env cell tnum tnum_iter _).

   Definition doit (p: program) : alarm_mon unit :=
    let ge := Genv.globalenv p in
    let mem_dom := mem_cshm_dom M ge max_concretize in
    iter_program _ _ unroll p
      (if trace then
         @debug_mem_dom _ _ mem_dom (TToString ge)
                        (@bot_lift_toString _ (IterTToString ge)) verbose
       else mem_dom).

End S.

Local Unset Elimination Schemes.
Inductive num_dom_kind :=
| ZIntervals
| Octagons
| Poly
.

Existing Instance ACTreeDom.toString.

Section VA.

  Context (kind: num_dom_kind)
          (max_concretize: N)
          (trace verbose: bool)
          (unroll: nat).

  Definition vanalysis (p: program) (* : alarm_mon unit  *):=
    let CS : ToString _ := CellToString (Genv.globalenv p) in
    match kind return _ with
    | ZIntervals =>
      let t_dom := reduced_product
        (AbTyps.t * (AbVarExpEq.t * (ZCongruences.t * IdealIntervalsNonrel.t))) _
      in
      let dom := idealenv_to_machineenv t_dom _ in
      let dom := if trace then debug_ab_machine_env verbose dom else dom in
      doit max_concretize trace verbose unroll dom p
    | Octagons =>
      let t_dom := reduced_product
                     (AbTyps.t * (AbVarExpEq.t * (ZCongruences.t *
                     (IdealIntervalsNonrel.t * (AbLinearize.t * Octagons.t)))))%type
                     _
      in
      let dom := idealenv_to_machineenv t_dom _ in
      let dom := if trace then debug_ab_machine_env verbose dom else dom in
      doit max_concretize trace verbose unroll dom p
    | Poly =>
      let t_dom := reduced_product
        (AbTyps.t * (AbVarExpEq.t * (ZCongruences.t * (IdealIntervalsNonrel.t * PolAdomZ.A)))) _
      in
      let dom := idealenv_to_machineenv t_dom _ in
      let dom := if trace then debug_ab_machine_env verbose dom else dom in
      doit max_concretize trace verbose unroll dom p
    end.

  Theorem vanalysis_correct :
    forall prog, vanalysis prog = (tt, nil) ->
    forall tr, program_behaves (semantics prog) (Goes_wrong tr) ->
    False.
  Proof.
    unfold vanalysis, doit. intros.
    destruct kind, trace, verbose; eapply iter_program_sound; eauto;
    match goal with
      | |- mem_dom_spec _ (debug_mem_dom _ _) _ _ => eapply debug_mem_dom_sound
      | _ => idtac
    end;
    eapply mem_cshm_dom_correct.
  Qed.

  Local Notation "⟦ p ⟧" := (program_behaves (semantics p)).
  Import Util.

  Corollary vanalysis_sound:
    ∀ p, vanalysis p = (tt, nil) →
         ⟦p⟧ ⊆ not_wrong.
  Proof.
    intros p H (); try easy.
    intros t H0. exact (vanalysis_correct p H _ H0).
  Qed.

End VA.
