(* *********************************************************************)
(*                                                                     *)
(*              The Verasco static analyzer                            *)
(*                                                                     *)
(*          Vincent Laporte, Université de Rennes 1                    *)
(*          Jacques-Henri Jourdan, INRIA Paris-Rocquencourt            *)
(*                                                                     *)
(*  Copyright Institut National de Recherche en Informatique et en     *)
(*  Automatique and Université de Rennes 1.  All rights reserved.      *)
(*  This file is distributed under the terms of the GNU General        *)
(*  Public License as published by the Free Software Foundatio,        *)
(*  either version 2 of the License, or (at your option) any later     *)
(*  version.  This file is also distributed under the terms of the     *)
(*  INRIA Non-Commercial License Agreement.                            *)
(*                                                                     *)
(* *********************************************************************)

Require Import
  Utf8 String Coqlib ToString
  DebugCminor
  Integers Floats AST Csharpminor.

Local Open Scope string_scope.

Definition string_of_const (c: constant) : string :=
  match c with
  | Ointconst i => to_string i
  | Olongconst i => to_string i
  | Ofloatconst f => to_string f
  | Osingleconst f => to_string f
  end.

Instance ConstantToString : ToString constant := string_of_const.

Fixpoint string_of_expr (e: expr) : string :=
  match e with
  | Evar i => "var " ++ to_string i
  | Eaddrof i => "&" ++ to_string i
  | Econst c => to_string c
  | Eunop op e => to_string op ++ string_of_expr e
  | Ebinop op e e' => "(" ++ string_of_expr e ++ to_string op ++ string_of_expr e' ++ ")"
  | Eload κ e => to_string κ ++ "[" ++ string_of_expr e ++ "]"
  end.

Instance ExprToString : ToString expr := string_of_expr.
